using System;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Windows.Forms;
using ScsMinimapper.Scs;
using ScsMinimapper.Dialogs;

namespace ScsMinimapper {
	public partial class ScsMinimapper : Form {
		private BatchDialog batchDialog;

		public ScsMinimapper() {
			batchDialog = new BatchDialog();
			InitializeComponent();
		}

		private void SetTitle(string title) {
			if (string.IsNullOrEmpty(title)) {
				Text = "SCS Minimapper";
			} else {
				Text = title + " - SCS Minimapper";
			}
		}

		private void openMenuItem_Click(object sender, EventArgs e) {
			DialogResult result = openFileDialog.ShowDialog();
			if (result == DialogResult.OK) {
				SetTitle(null);
				saveMenuItem.Enabled = false;
				ScsFile scs = new ScsFile(openFileDialog.FileName);
				busyLabel.Visible = true;
				backgroundWorker.RunWorkerAsync(scs);
			}
		}

		private void backgroundWorker_DoWork(object sender, DoWorkEventArgs e) {
			ScsFile scs = e.Argument as ScsFile;
			if (scs == null) {
				return;
			}
			scs.ProcessFile();
			e.Result = scs;
		}

		private void backgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e) {
			busyLabel.Visible = false;
			ScsFile scs = e.Result as ScsFile;
			if (scs == null) {
				// Theoretically shouldn't occur..
				minimapBox.Image = null;
				MessageBox.Show("The SCS file could not be opened", "Error loading minimap");
			} else if (scs.Image == null) {
				minimapBox.Image = null;
				MessageBox.Show("The minimap could not be loaded:\r\n"
					+ scs.ErrorMessage, "Error loading minimap");
			} else {
				SetTitle(scs.Name);
				minimapBox.Image = scs.Image;
				saveMenuItem.Enabled = true;
			}
		}

		private void saveMenuItem_Click(object sender, EventArgs e) {
			// Basic sanity check
			if (minimapBox.Image == null) {
				MessageBox.Show("This shouldn't happen! No image to save!");
				return;
			}

			// Suggest a file name
			FileInfo fi = new FileInfo(openFileDialog.FileName);
			saveFileDialog.FileName = Path.ChangeExtension(fi.Name, ".png");

			// Show the dialog
			DialogResult result = saveFileDialog.ShowDialog();
			if (result == DialogResult.OK) {
				// Save the file
				minimapBox.Image.Save(saveFileDialog.FileName, ImageFormat.Png);
			}
		}

		private void batchMenuItem_Click(object sender, EventArgs e) {
			batchDialog.ShowDialog();
		}

		private void exitMenuItem_Click(object sender, EventArgs e) {
			Close();
		}

		private void helpMenuItem_Click(object sender, EventArgs e) {
			HelpDialog dialog = new HelpDialog();
			dialog.ShowDialog();
		}

		private void licenceMenuItem_Click(object sender, EventArgs e) {
			LicenceDialog dialog = new LicenceDialog();
			dialog.ShowDialog();
		}

		private void aboutMenuItem_Click(object sender, EventArgs e) {
			AboutDialog dialog = new AboutDialog();
			dialog.ShowDialog();
		}
	}
}
